/*
 * VCNL4035X01_ALS.c
 *
 * Created  : 9 November 2020
 * Modified : 1 February 2021
 * Author   : HWanyusof
 * Version	: 4
 */ 

#include "VCNL4035X01_Prototypes.h"
#include "VCNL4035X01.h"
#include "I2C_Functions.h"
#include "VCNL4035X01_Application_Library.h"


extern int I2C_Bus;

//****************************************************************************************************
//*****************************************Sensor API*************************************************

//Turn On/Off the ALS Sensor
void VCNL4035X01_SET_ALS_SD(Byte SD_Bit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_1;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_ALS_SD_EN|VCNL4035X01_ALS_SD_DIS))|SD_Bit;
	VCNL4035X01_Data.WData[1] = VCNL4035X01_Data.RData[1];
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Integration Time
void VCNL4035X01_SET_ALS_IT(Byte IntTime)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_1;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_ALS_IT_50MS|VCNL4035X01_ALS_IT_100MS|VCNL4035X01_ALS_IT_200MS|VCNL4035X01_ALS_IT_400MS|VCNL4035X01_ALS_IT_800MS))|IntTime;
	VCNL4035X01_Data.WData[1] = VCNL4035X01_Data.RData[1];
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Typical Dynamic Range
void VCNL4035X01_SET_ALS_HD(Byte HD)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_1;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_ALS_HD_X1|VCNL4035X01_ALS_HD_X2))|HD;
	VCNL4035X01_Data.WData[1] = VCNL4035X01_Data.RData[1];
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Persistence
void VCNL4035X01_SET_ALS_PERS(Byte Pers)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_1;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0]=(VCNL4035X01_Data.RData[0]&~(VCNL4035X01_ALS_PERS_1|VCNL4035X01_ALS_PERS_2|VCNL4035X01_ALS_PERS_4|VCNL4035X01_ALS_PERS_8))|Pers;
	VCNL4035X01_Data.WData[1] = VCNL4035X01_Data.RData[1];
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable Interrupt
void VCNL4035X01_SET_ALS_Interrupt(Byte Interrupt)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_1;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0]=(VCNL4035X01_Data.RData[0]&~(VCNL4035X01_ALS_INT_EN|VCNL4035X01_ALS_INT_DIS))|Interrupt;
	VCNL4035X01_Data.WData[1] = VCNL4035X01_Data.RData[1];
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the ALS Sensitivity
void VCNL4035X01_SET_ALS_NS(Byte ALS_NS)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_2;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0]=VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_ALS_NS_X1|VCNL4035X01_ALS_NS_X2))|ALS_NS;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Turn On/Off the White Channel
void VCNL4035X01_SET_WHITE_SD(Byte WhiteSD)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_2;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0]=VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_ALS_WHITE_SD_EN|VCNL4035X01_ALS_WHITE_SD_DIS))|WhiteSD;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Read the ALS Data
Word VCNL4035X01_GET_ALS_Data()
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_DATA;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	return ((VCNL4035X01_Data.RData[1]<<8)|VCNL4035X01_Data.RData[0]);
}

//Read the White Channel Data
Word VCNL4035X01_GET_White_Data()
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_WHITE_DATA;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	return ((VCNL4035X01_Data.RData[1]<<8)|VCNL4035X01_Data.RData[0]);
}

//Set the Low Threshold
void VCNL4035X01_SET_ALS_LowThreshold(Word LowThreshold)
{
	Byte LowByte = LowThreshold;
	Byte HighByte = LowThreshold>>8;
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_THDL;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = LowByte;
	VCNL4035X01_Data.WData[1] = HighByte;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the High Threshold
void VCNL4035X01_SET_ALS_HighThreshold(Word HighThreshold)
{
	Byte LowByte = HighThreshold;
	Byte HighByte = HighThreshold>>8;
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_THDH;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = LowByte;
	VCNL4035X01_Data.WData[1] = HighByte;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Read the ALS Interrupt Flag
Word VCNL4035X01_GET_ALS_Interrupt()
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_INT_FLAG;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	return (VCNL4035X01_Data.RData[1]);
}