/*
 * VCNL4035X01_Application_Library.c
 *
 * Created  : 9 November 2020
 * Modified : 1 February 2021
 * Author   : HWanyusof
 * Version	: 4
 */ 

#include "VCNL4035X01_Prototypes.h"
#include "VCNL4035X01.h"
#include "I2C_Functions.h"
#include "VCNL4035X01_Application_Library.h"

extern int I2C_Bus;

//****************************************************************************************************
//***************************************Application API**********************************************

//Determine Sensitivity From ALS IT
float VCNL4035X01_CAL_Sensitivity(Byte ALS_IT)
{
    float Sensitivity;

	if (ALS_IT == VCNL4035X01_ALS_IT_50MS) {Sensitivity = 0.064;}
	if (ALS_IT == VCNL4035X01_ALS_IT_100MS) {Sensitivity = 0.032;}
	if (ALS_IT == VCNL4035X01_ALS_IT_200MS) {Sensitivity = 0.016;}
	if (ALS_IT == VCNL4035X01_ALS_IT_400MS) {Sensitivity = 0.008;}
	if (ALS_IT == VCNL4035X01_ALS_IT_800MS) {Sensitivity = 0.004;}
	
	return(Sensitivity);
}

//Calculate the Als (Lux)
float VCNL4035X01_CAL_Lux(float Sensitivity, float Count)
{
    float Lux;
    Lux = Sensitivity*Count;
    return(Lux);
}

//Get Delay for Measurement
int VCNL4035X01_GET_Delay(Byte ALS_IT)
{
	int Delay;
	
	//Delay for ALS
	//Delay = Delay from IT + Circuit (~10ms)
	if (ALS_IT == VCNL4035X01_ALS_IT_50MS) {Delay = 60;}
	if (ALS_IT == VCNL4035X01_ALS_IT_100MS) {Delay = 110;}
	if (ALS_IT == VCNL4035X01_ALS_IT_200MS) {Delay = 210;}
	if (ALS_IT == VCNL4035X01_ALS_IT_400MS) {Delay = 410;}
	if (ALS_IT == VCNL4035X01_ALS_IT_800MS) {Delay = 810;}
	
	return Delay;
}

//Reset the Sensor to the default value
void Reset_Sensor()
{   
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_CONF_1;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x01;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_THDL;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x00;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ALS_THDH;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x00;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_1;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x01;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x00;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
    VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CANC;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x00;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_THDL;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x00;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_THDH;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x00;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
	
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_INT_FLAG;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = 0x00;
	VCNL4035X01_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Print the Register Value (Only supported for PSoc Cypress, STM32 and Arduino supported MCU)
void Print_Sensor(char mode_ps,char mode_als,Byte ALS_IT)
{
	Word value;
	Word Data1,Data2,Data3;
    bool Gesture_Data_Ready;
	int Delay;
	float Sensitivity;
	float Lux;
    
    #define TRANSMIT_BUFFER_SIZE  128
	char   TransmitBuffer[TRANSMIT_BUFFER_SIZE];
	char   TransmitBuffer2[TRANSMIT_BUFFER_SIZE];
	
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer2,"*************************************************** \r\n");
		CDC_PutString(TransmitBuffer2);
	}
	CyDelay(50);
	
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer2,">>>>>>>>>>>>>>>>>>>>>>>ALS<<<<<<<<<<<<<<<<<<<<<<<<< \r\n");
		CDC_PutString(TransmitBuffer2);
	}
	CyDelay(50);
	
	//Print the Reg 0x00 (ALS_CONF1 and ALS_CONF2)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_ALS_CONF_1);
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x00 (ALS_CONF) Val : 0x%x \r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Print the Reg 0x01 (ALS_THDH)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_ALS_THDH);
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x01 (ALS_THDH) Val : %d Counts\r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Print the Reg 0x02 (ALS_THDL)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_ALS_THDL);
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x02 (ALS_THDL) Val : %d Counts\r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Get Delay for ALS Measurement
	Delay = VCNL4035X01_GET_Delay(ALS_IT);
	
	//Delay of IT ms + other Circuit Delay (~10ms)
	CyDelay(Delay);
	
	//Print ALS Data
	if(mode_als == 'L')
	{
		//Find Sensitivity
		Sensitivity = VCNL4035X01_CAL_Sensitivity(ALS_IT);
		
		//Print the Sensitivity
		if(CDC_CDCIsReady() != 0u)
		{   
			ftoa(Sensitivity,TransmitBuffer, 6); 
			sprintf(TransmitBuffer2,">>>>>>>Sensitivity : %s lx/Count<<<<<<<<\r\n",TransmitBuffer);
			CDC_PutString(TransmitBuffer2);
		}
		CyDelay(50);
		
		//Print the ALS Data in DEC
		value = VCNL4035X01_GET_ALS_Data();
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>ALS Data : %d Counts<<<<<<<<\r\n",value);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
		
		//Calculate Lux
		Lux = VCNL4035X01_CAL_Lux(Sensitivity, value);
		
		//Print the ALS Lux in DEC
		if(CDC_CDCIsReady() != 0u)
		{   
			ftoa(Lux,TransmitBuffer, 5); 
			sprintf(TransmitBuffer2,">>>>>>>ALS Lux : %s lx<<<<<<<<\r\n",TransmitBuffer);
			CDC_PutString(TransmitBuffer2);
		}
		CyDelay(50);
	}
	
	//Print White Data
	if(mode_als == 'W')
	{
		//Print the White Data in DEC
		value = VCNL4035X01_GET_White_Data();
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>White Data : %d Counts<<<<<<<<\r\n",value);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
	}
	
	//Print the Interrupt Flag
	if((mode_ps != 'S') && (mode_ps != 'A')&&(mode_ps != 'G'))
	{
		value = VCNL4035X01_GET_ALS_Interrupt();
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Interrupt Flag : 0x%x<<<<<<<< \r\n",value);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
	}
	
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer2,"*************************************************** \r\n");
		CDC_PutString(TransmitBuffer2);
	}
	CyDelay(50);
	
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer2,">>>>>>>>>>>>>>>>>>>>>>>>PS<<<<<<<<<<<<<<<<<<<<<<<<< \r\n");
		CDC_PutString(TransmitBuffer2);
	}
	CyDelay(50);
	
	//Print the Reg 0x03 (PS_CONF1 and PS_CONF2)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_CONF_1);
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x03 (PS_CONF1) Val : 0x%x \r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Print the Reg 0x04 (PS_CONF3 and PS_MS)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_CONF_3);
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x04 (PS_CONF3) Val : 0x%x \r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Print the Reg 0x05 (PS_CANC)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_CANC);
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x05 (PS_CANC) Val : %d Counts \r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Print the Reg 0x06 (PS_THDL)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_THDL);      
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x06 (PS_THDL) Val : %d Counts \r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Print the Reg 0x07 (PS_THDH)
	value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_THDH);
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer,">>>>>>>Reg 0x07 (PS_THDH) Val : %d Counts \r\n",value);
		CDC_PutString(TransmitBuffer);
	}
	CyDelay(50);
	
	//Print Proximity Data (Only For Auto Mode, AF and Low Power Mode)
	if((mode_ps == 'S') ||(mode_ps == 'A'))
	{
		
		//Set Trigger for AF Mode + Associated delay due to IT
		if(mode_ps == 'A')
		{
			//Set trigger to start a measurement
			VCNL4035X01_SET_PS_TRIG(VCNL4035X01_PS_TRIG_EN);
			
			//Delay of PS Measurement + other Circuit Delay
			CyDelay(50);
		}
		
		//Delay for Auto and Low Power Mode + Associated delay due to IT
		if(mode_ps == 'S')
		{	
			//Delay of PS Measurement + other Circuit Delay
			CyDelay(50);
		}
		
		//Print the Proximity Data (IRED 1)
		value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_DATA_1);
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Proximity Data 1 : %d Counts<<<<<<<< \r\n",value);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
		
		//Print the Proximity Data (IRED 2)
		value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_DATA_2);
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Proximity Data 2 : %d Counts<<<<<<<< \r\n",value);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
		
		//Print the Proximity Data (IRED 3)
		value = VCNL4035X01_READ_Reg(VCNL4035X01_PS_DATA_3);
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Proximity Data 3 : %d Counts<<<<<<<< \r\n",value);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
		
		//Print the Interrupt Flag
		value = VCNL4035X01_GET_PS_Interrupt();
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Interrupt Flag : 0x%x<<<<<<<< \r\n",value);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
		
	}
	
	//Print Gesture Data (Only Gesture Mode)
	if(mode_ps == 'G')
	{	
		//Set trigger to start a measurement
		VCNL4035X01_SET_PS_TRIG(VCNL4035X01_PS_TRIG_EN);
		
		//Delay of PS Measurement + other Circuit Delay
		CyDelay(50);
		
		//Print the Gesture Data Ready Interrupt Flag
		Gesture_Data_Ready = VCNL4035X01_GET_Gesture_Data_Ready_Flag();
		if(CDC_CDCIsReady() != 0u)
		{   
			//Need to convert to int since sprintf has no bool specifier
			sprintf(TransmitBuffer,">>>>>>>Gesture Data Ready INT Flag : 0x%x<<<<<<<< \r\n",(int)Gesture_Data_Ready);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
		
		//Read Gesture Data
		VCNL4035X01_GET_Gesture_Mode_Data(&Data1, &Data2, &Data3);

		//Print the Proximity Data (IRED 1)
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Proximity Data 1 : %d Counts<<<<<<<< \r\n",Data1);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);

		//Print the Proximity Data (IRED 2)
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Proximity Data 2 : %d Counts<<<<<<<< \r\n",Data2);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);

		//Print the Proximity Data (IRED 3)
		if(CDC_CDCIsReady() != 0u)
		{   
			sprintf(TransmitBuffer,">>>>>>>Proximity Data 3 : %d Counts<<<<<<<< \r\n",Data3);
			CDC_PutString(TransmitBuffer);
		}
		CyDelay(50);
		
	}
	
	if(CDC_CDCIsReady() != 0u)
	{   
		sprintf(TransmitBuffer2,"*************************************************** \r\n");
		CDC_PutString(TransmitBuffer2);
	}

	CyDelay(2000);
}

//Reverses a string 'str' of length 'len' 
void reverse(char* str, int len) 
{ 
    int i = 0, j = len - 1, temp; 
    while (i < j) { 
        temp = str[i]; 
        str[i] = str[j]; 
        str[j] = temp; 
        i++; 
        j--; 
    } 
} 
  
//Converts a given integer x to string str[].  
//d is the number of digits required in the output.  
//If d is more than the number of digits in x,  
//then 0s are added at the beginning. 
int intToStr(int x, char str[], int d) 
{ 
    int i = 0; 

    while (x) 
	{ 
		//Store and convert int to char (Valid for single digit)
        str[i++] = (x % 10) + '0'; 
        x = x / 10; 
    } 
  
    //If number of digits required is more, then 
    //add 0s at the beginning 
    while (i < d) str[i++] = '0'; 
	
	//Reverse the string characters in the array str
    reverse(str, i);
	
	//Place the null character at the end of the array
    str[i] = '\0'; 
	
	//Return the position i
    return i; 
} 

//Converts a floating-point/double number to a string. 
void ftoa(float n, char* res, int afterpoint) 
{ 
    //Extract integer part 
    int ipart = (int)n; 
  
    //Extract decimal part 
    float fpart = n - (float)ipart; 
  
    //Convert integer part to string and the function returns the position after the interger
    int i = intToStr(ipart, res, 0); 
  
    //Check for display option after point 
    if (afterpoint != 0) 
	{ 
		//Add dot after the integer part
        res[i] = '.'; 
  
        //Multiply decimal part by 10^decimal point
        fpart = fpart* pow(10, afterpoint); 
		
		//Convert decimal part to string
        intToStr((int)fpart, res + i + 1, afterpoint); 
    }
}
