/*
 * VCNL4035X01_PS.c
 *
 * Created  : 9 November 2020
 * Modified : 1 Februray 2021
 * Author   : HWanyusof
 * Version	: 4
 */ 

#include "VCNL4035X01_Prototypes.h"
#include "VCNL4035X01.h"
#include "I2C_Functions.h"
#include "VCNL4035X01_Application_Library.h"

extern int I2C_Bus;

//****************************************************************************************************
//*****************************************Sensor API*************************************************

//Set the PS On/Off Duty Ratio
void VCNL4035X01_SET_PS_DUTY(Byte DutyRatio)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_1;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_DR_1_40|VCNL4035X01_PS_DR_1_80|VCNL4035X01_PS_DR_1_160|VCNL4035X01_PS_DR_1_320))|DutyRatio;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Persistence
void VCNL4035X01_SET_PS_PERS(Byte Pers)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_1;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_PERS_1|VCNL4035X01_PS_PERS_2|VCNL4035X01_PS_PERS_3|VCNL4035X01_PS_PERS_4))|Pers;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Integration Time
void VCNL4035X01_SET_PS_IT(Byte IntegrationTime)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_1;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_IT_1T|VCNL4035X01_PS_IT_1_5T|VCNL4035X01_PS_IT_2T|VCNL4035X01_PS_IT_2_5T|VCNL4035X01_PS_IT_3T|VCNL4035X01_PS_IT_3_5T|VCNL4035X01_PS_IT_4T|VCNL4035X01_PS_IT_8T))|IntegrationTime;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Turn the PS Sensor On/Off
void VCNL4035X01_SET_PS_SD(Byte SDBit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_1;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_SD_DIS|VCNL4035X01_PS_SD_EN))|SDBit;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable the Gesture Interrupt 
void VCNL4035X01_SET_GESTURE_INT(Byte gesture_int)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_2;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_GESTURE_INT_EN|VCNL4035X01_PS_GESTURE_INT_DIS))|gesture_int;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable the Gesture Mode 
void VCNL4035X01_SET_GESTURE_MODE(Byte gesture_mode)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_2;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_GESTURE_MODE_EN|VCNL4035X01_PS_GESTURE_MODE_DIS))|gesture_mode;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Gain
void VCNL4035X01_SET_PS_Gain(Byte Gain)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_2;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_GAIN_2_STEP|VCNL4035X01_PS_GAIN_SINGLEx8|VCNL4035X01_PS_GAIN_SINGLEx1))|Gain;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the PS Output size (12/16Bits)
void VCNL4035X01_SET_PS_HD(Byte HDBit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_2;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_HD_12Bits|VCNL4035X01_PS_HD_16Bits))|HDBit;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Sensitivity
void VCNL4035X01_SET_PS_NS(Byte NSBit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_2;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_NS_2STEP_MODE_X4|VCNL4035X01_PS_NS_2STEP_MODE))|NSBit;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Interrupt Mode
void VCNL4035X01_SET_PS_INT(Byte InterruptMode)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_2;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = VCNL4035X01_Data.RData[0];
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_INT_DIS|VCNL4035X01_PS_INT_CLOSE|VCNL4035X01_PS_INT_AWAY|VCNL4035X01_PS_INT_CLOSE_AWAY))|InterruptMode;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable Low Power Mode
void VCNL4035X01_SET_PS_LED_I_LOW(Byte LOW_I_Bit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_I_LOW_DIS|VCNL4035X01_PS_I_LOW_EN))|LOW_I_Bit;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Select IRED input to be Driven by the Internal Driver
void VCNL4035X01_SET_PS_IRED_select(Byte ired)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_IRED_1|VCNL4035X01_PS_IRED_2|VCNL4035X01_PS_IRED_3))|ired;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable Smart Persistence
void VCNL4035X01_SET_PS_SMART_PERS(Byte Pers)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_SMART_PERS_DIS|VCNL4035X01_PS_SMART_PERS_EN))|Pers;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable Active Force or Auto Mode
void VCNL4035X01_SET_PS_AF(Byte AF_Bit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_AUTO|VCNL4035X01_PS_AF_EN))|AF_Bit;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable Trigger for AF Mode
void VCNL4035X01_SET_PS_TRIG(Byte TriggerBit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_TRIG_DIS|VCNL4035X01_PS_TRIG_EN))|TriggerBit;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Proximity Mode (Normal/Logic)
void VCNL4035X01_SET_PS_MS(Byte MS_Bit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_MS_NORMAL|VCNL4035X01_PS_MS_LOGIC))|MS_Bit;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Enable/Disable Sunlight Cancellation
void VCNL4035X01_SET_PS_SC_EN(Byte SunlightBit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_3;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]&~(VCNL4035X01_PS_SC_DIS|VCNL4035X01_PS_SC_EN))|SunlightBit;
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]);
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Sunlight Cancellation Current
void VCNL4035X01_SET_PS_SC_CUR(Byte SC_Current)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_MS;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]);
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_SC_CURx1|VCNL4035X01_PS_SC_CURx2|VCNL4035X01_PS_SC_CURx4|VCNL4035X01_PS_SC_CURx8))|SC_Current;
	WriteI2C_Bus(&VCNL4035X01_Data);

}

//Set the Sunlight Capability
void VCNL4035X01_SET_PS_SP(Byte SP_Bit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_MS;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]);
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_SPx1|VCNL4035X01_PS_SPx1_5))|SP_Bit;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Output of Sunlight Protect Mode 
void VCNL4035X01_SET_PS_SPO(Byte SPO_Bit)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_MS;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]);
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_SPO_00|VCNL4035X01_PS_SPO_FF))|SPO_Bit;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the LED current
void VCNL4035X01_SET_PS_LED_I(Byte IRED_Current)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CONF_MS;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	VCNL4035X01_Data.WData[0] = (VCNL4035X01_Data.RData[0]);
	VCNL4035X01_Data.WData[1] = (VCNL4035X01_Data.RData[1]&~(VCNL4035X01_PS_LED_I_50mA|VCNL4035X01_PS_LED_I_75mA|VCNL4035X01_PS_LED_I_100mA|VCNL4035X01_PS_LED_I_120mA|VCNL4035X01_PS_LED_I_140mA|VCNL4035X01_PS_LED_I_160mA|VCNL4035X01_PS_LED_I_180mA|VCNL4035X01_PS_LED_I_200mA))|IRED_Current;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the PS Cancellation
void VCNL4035X01_SET_PS_CANC(Word CancelValue)
{
	Byte LowByte = CancelValue;
	Byte HighByte = CancelValue>>8;
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_CANC;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = LowByte;
	VCNL4035X01_Data.WData[1] = HighByte;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the Low Threshold
void VCNL4035X01_SET_PS_LowThreshold(Word LowThreshold)
{
	Byte LowByte = LowThreshold;
	Byte HighByte = LowThreshold>>8;
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_THDL;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = LowByte;
	VCNL4035X01_Data.WData[1] = HighByte;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Set the High Threshold
void VCNL4035X01_SET_PS_HighThreshold(Word HighThreshold)
{
	Byte LowByte = HighThreshold;
	Byte HighByte = HighThreshold>>8;
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_PS_THDH;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	VCNL4035X01_Data.WData[0] = LowByte;
	VCNL4035X01_Data.WData[1] = HighByte;
	WriteI2C_Bus(&VCNL4035X01_Data);
}

//Read the PS Data
Word VCNL4035X01_GET_PS_Data(Byte Reg)
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = Reg;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	return (VCNL4035X01_Data.RData[1]<<8|VCNL4035X01_Data.RData[0]);
}

//Read the PS Interruppt Flag
Word VCNL4035X01_GET_PS_Interrupt()
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_INT_FLAG;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	return (VCNL4035X01_Data.RData[1]);
}

//Read Register value
Word VCNL4035X01_READ_Reg(Byte Reg)
{
	Word RegValue;
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = Reg;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	RegValue = VCNL4035X01_Data.RData[1]<<8|VCNL4035X01_Data.RData[0];
	return RegValue;
}

//Read Gesture Mode Data
void VCNL4035X01_GET_Gesture_Mode_Data(Word *Data1, Word *Data2, Word *Data3)
{
	struct GestureTransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus_Gesture_Mode(&VCNL4035X01_Data);
	*Data1 = VCNL4035X01_Data.RData[1]<<8|VCNL4035X01_Data.RData[0];
	*Data2 = VCNL4035X01_Data.RData[3]<<8|VCNL4035X01_Data.RData[2];
	*Data3 = VCNL4035X01_Data.RData[5]<<8|VCNL4035X01_Data.RData[4];
}

//Read the Gesture Data Ready Flag
bool VCNL4035X01_GET_Gesture_Data_Ready_Flag()
{
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_INT_FLAG;
    VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	return (VCNL4035X01_Data.RData[1]>>7);
}

//Read Device ID
Word VCNL4035X01_GET_ID()
{
	Word RegValue;
	struct TransferData VCNL4035X01_Data;
	VCNL4035X01_Data.Slave_Address = VCNL4035X01_SlaveAddress;
	VCNL4035X01_Data.RegisterAddress = VCNL4035X01_ID;
	VCNL4035X01_Data.Select_I2C_Bus = I2C_Bus;
	ReadI2C_Bus(&VCNL4035X01_Data);
	RegValue = VCNL4035X01_Data.RData[1]<<8|VCNL4035X01_Data.RData[0];
	return RegValue;
}
