/*
 * Main.c
 *
 *  Created on  : 15 December 2020
 *  Modified on : 1 February 2021
 *  Author      : HWanyusof
 *  Version     : 4 
 */

#include "VCNL4035X01.h"
#include "VCNL4035X01_Prototypes.h"
#include "typedefinition.h"
#include "VCNL4035X01_Application_Library.h" 
#include "project.h"

//Global Variables
int value;
char mode_ps;
char mode_als;

//Switch the I2C Bus number here
int I2C_Bus = 2;

//Variables for IT
Byte ALS_IT = VCNL4035X01_ALS_IT_50MS;//For ALS and White Mode 
Byte PS_IT = VCNL4035X01_PS_IT_2T;//For Auto, AF, Low Power and Gesture Mode

//Variable for Offset Value
int CalibValue = 0;
int OffsetValue = 0;
int AverageCount = 10; //Change the Averagecount to the needed number of offset measurement
int Max_Offset = 0;

//Variable for IRED channel
int IRED_Channel = VCNL4035X01_PS_IRED_3;
int IRED_Channel_Offset_Read = VCNL4035X01_PS_DATA_3;

/* Function Prototypes */
void AF_Mode();
void Auto_Mode();
void Gesture_Mode();
void Low_Power_Mode();
void White_Mode();
void ALS_Mode();

int main(void)
{
    /* Enable global interrupts. */
    CyGlobalIntEnable;
    
    /* Place your initialization/startup code here (e.g. MyInst_Start()) */
    I2C_1_Start();
    CDC_Start(0,CDC_5V_OPERATION);
    CDC_CDC_Init();

    /* Comment/Uncomment this section if Reset is needed/not needed */
    Reset_Sensor();
    
    /* Choose a desired proximity mode and comment the rest*/
    Auto_Mode(); //Also known as Self-Timed Mode
    //AF_Mode(); //Also known Continuous Forced Mode (Could also be used for single forced mode)
    //Gesture_Mode();
    //Low_Power_Mode(); //Auto/Self-Timed Mode with 1/10 of normal LED current

    /* Choose a desired light sensing mode (ALS or White Channel) and comment the rest*/
    ALS_Mode();
    //White_Mode();
    
    /* Loop to run Application */
    for(;;)
    {
        /* Comment/Uncomment this section if Print is needed/not needed */
        Print_Sensor(mode_ps,mode_als,ALS_IT);
        
        /* Place your application code here. */
    }
}

void White_Mode()
{
    //1.)Initialization
    //Switch the ALS off
    VCNL4035X01_SET_ALS_SD(VCNL4035X01_ALS_SD_DIS);
    //Disable White Channel
    VCNL4035X01_SET_WHITE_SD(VCNL4035X01_ALS_WHITE_SD_DIS);

    //2.) Setting up White Channel
    //Can use ALS channel parameters
    //ALS_CONF1
    //Enable/Disable Interrupt
    VCNL4035X01_SET_ALS_Interrupt(VCNL4035X01_ALS_INT_EN);
    //Set the Persistence of the Ambient Light Sensor
    VCNL4035X01_SET_ALS_PERS(VCNL4035X01_ALS_PERS_1);
    //Set the Dynamic Range
    VCNL4035X01_SET_ALS_HD(VCNL4035X01_ALS_HD_X1);
    //Set the ALS Integration Time
    VCNL4035X01_SET_ALS_IT(ALS_IT);

    //ALS_CONF2
    //Set the ALS Sensitivity
    VCNL4035X01_SET_ALS_NS(VCNL4035X01_ALS_NS_X2);

    //ALS_THDH
    //Set the ALS Interrupt Higher Threshold
    VCNL4035X01_SET_ALS_HighThreshold(1000);

    //ALS_THDL
    //Set the ALS Interrupt Lower Threshold
    VCNL4035X01_SET_ALS_LowThreshold(500);

    //3.) Switch on the White and ALS Channel
    VCNL4035X01_SET_ALS_SD(VCNL4035X01_ALS_SD_EN);
    VCNL4035X01_SET_WHITE_SD(VCNL4035X01_ALS_WHITE_SD_EN);
    
    //Tag mode to be White Mode
    mode_als = 'W';
    
    while (I2C_1_MasterStatus()!=I2C_1_MODE_COMPLETE_XFER){}
    CyDelay(1000);
}

void ALS_Mode()
{
    //1.)Initialization
    //Switch the ALS off
    VCNL4035X01_SET_ALS_SD(VCNL4035X01_ALS_SD_DIS);
    //En/Dis White Channel (Disable for normal ALS wavelength
    VCNL4035X01_SET_WHITE_SD(VCNL4035X01_ALS_WHITE_SD_DIS);

    //2.) Setting up ALS
    //ALS_CONF1
    //Enable/Disable Interrupt
    VCNL4035X01_SET_ALS_Interrupt(VCNL4035X01_ALS_INT_EN);
    //Set the Persistence of the Ambient Light Sensor
    VCNL4035X01_SET_ALS_PERS(VCNL4035X01_ALS_PERS_1);
    //Set the Dynamic Range
    VCNL4035X01_SET_ALS_HD(VCNL4035X01_ALS_HD_X1);
    //Set the ALS Integration Time
    VCNL4035X01_SET_ALS_IT(ALS_IT);

    //ALS_CONF2
    //Set the ALS Sensitivity
    VCNL4035X01_SET_ALS_NS(VCNL4035X01_ALS_NS_X2);

    //ALS_THDH
    //Set the ALS Interrupt Higher Threshold
    VCNL4035X01_SET_ALS_HighThreshold(1000);

    //ALS_THDL
    //Set the ALS Interrupt Lower Threshold
    VCNL4035X01_SET_ALS_LowThreshold(500);

    //3.) Switch on the ALS
    VCNL4035X01_SET_ALS_SD(VCNL4035X01_ALS_SD_EN);
    
    //Tag mode to be ALS Mode
    mode_als = 'L';
    
    while (I2C_1_MasterStatus()!=I2C_1_MODE_COMPLETE_XFER){}
    CyDelay(1000);
}

void Low_Power_Mode()
{ 
    //1.)Initialization
    //Disable the PS
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_DIS);
    //Setting up Auto Mode
    VCNL4035X01_SET_PS_AF(VCNL4035X01_PS_AUTO);
    //PS_SD will be set before measuring the offset or after all of the parameters have been set up

    //2.) Setting up PS 
    //PS_CONF1
    //Set the PS IRED on / off duty ratio setting
    VCNL4035X01_SET_PS_DUTY(VCNL4035X01_PS_DR_1_40);
    //Set the Persistence
    VCNL4035X01_SET_PS_PERS(VCNL4035X01_PS_PERS_1);
    //Set the Integration Time
    VCNL4035X01_SET_PS_IT(PS_IT);

    //PS_CONF2
    //Enable/Disable the Gesture Interrupt 
    VCNL4035X01_SET_GESTURE_INT(VCNL4035X01_PS_GESTURE_INT_DIS);
    //Enable/Disable the Gesture Mode 
    VCNL4035X01_SET_GESTURE_MODE(VCNL4035X01_PS_GESTURE_MODE_DIS);
    //Set the Gain
    VCNL4035X01_SET_PS_Gain(VCNL4035X01_PS_GAIN_2_STEP);
    //Set the Output Bit Size
    VCNL4035X01_SET_PS_HD(VCNL4035X01_PS_HD_16Bits);
    //Set the Sensitivity Mode
    VCNL4035X01_SET_PS_NS(VCNL4035X01_PS_NS_2STEP_MODE_X4);
    //Set the Interrupt
    VCNL4035X01_SET_PS_INT(VCNL4035X01_PS_INT_CLOSE_AWAY);

    //PS_CONF3
    //Enable/Disable Low Current
    //When enabled -  1/10 of normal current,
    //with that the current is accordingly: 5 mA, 7.5 mA, 10 mA, 12 mA, 14 mA, 16 mA, 18 mA, 20 mA.
    //The current in VCNL4035X01_SET_PS_LED_I() still has to be set and the value will be 1/10 of that.  
    VCNL4035X01_SET_PS_LED_I_LOW(VCNL4035X01_PS_I_LOW_EN);
    //Select IRED input to be Driven by the Internal Driver
    VCNL4035X01_SET_PS_IRED_select(IRED_Channel);
    //Enable/Disable Smart Persistence
    VCNL4035X01_SET_PS_SMART_PERS(VCNL4035X01_PS_SMART_PERS_DIS);
    //Set Interrupt to Normal/Logic Mode
    VCNL4035X01_SET_PS_MS(VCNL4035X01_PS_MS_NORMAL);
    //Enable/Disable Sunlight Cancellation
    VCNL4035X01_SET_PS_SC_EN(VCNL4035X01_PS_SC_EN);
    //PS_AF has been set during initialization
    //PS_TRIG must be set before reading the proximity data

    //PS_MS
    //Set the Sunlight Cancellation Current
    VCNL4035X01_SET_PS_SC_CUR(VCNL4035X01_PS_SC_CURx1);
    //Set the Sunlight Capability
    VCNL4035X01_SET_PS_SP(VCNL4035X01_PS_SPx1);
    //Set the Output of Sunlight Protect Mode
    VCNL4035X01_SET_PS_SPO(VCNL4035X01_PS_SPO_00);
    //Set the LED Current
    VCNL4035X01_SET_PS_LED_I(VCNL4035X01_PS_LED_I_200mA);

    //3.) Switch the sensor on
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_EN);
    //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
    CyDelay(10);

    //4.) Threshold Setting and Offset Measurement
    for(int i=0;i<AverageCount;i++)
    {   
        CalibValue += VCNL4035X01_READ_Reg(IRED_Channel_Offset_Read);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
    }

    //Calculate the average of the offset measurement
    CalibValue /= AverageCount;

    //Check whether the offset value is too high to have been set in the cancellation register
    // 1T max offset value for the cancellation register is 2048
    if((PS_IT==VCNL4035X01_PS_IT_1T)||(PS_IT==VCNL4035X01_PS_IT_1_5T))Max_Offset = 2048;
    // 2T max offset value for the cancellation register is 4096
    if((PS_IT==VCNL4035X01_PS_IT_2T)||(PS_IT==VCNL4035X01_PS_IT_2_5T)||(PS_IT==VCNL4035X01_PS_IT_3T)||(PS_IT==VCNL4035X01_PS_IT_3_5T))Max_Offset = 4096;
    // 4T max offset value for the cancellation register is 8192
    if(PS_IT==VCNL4035X01_PS_IT_4T)Max_Offset = 8192;
    // 8T max offset value for the cancellation register is 16384
    if(PS_IT==VCNL4035X01_PS_IT_8T)Max_Offset = 16384;
    if(CalibValue>Max_Offset)
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500 + OffsetValue);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000 + OffsetValue);
    }
    else
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000);
    }

    //Tag mode to be Auto Mode/Self-timed
    mode_ps = 'S';

    while (I2C_1_MasterStatus()!=I2C_1_MODE_COMPLETE_XFER){}
    CyDelay(1000);
}

void Gesture_Mode()
{
    //1.)Initialization
    //Disable the PS
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_DIS);
    //Setting up AF Mode (Needed for Gesture Mode)
    VCNL4035X01_SET_PS_AF(VCNL4035X01_PS_AF_EN);

    //2.) Setting up PS 
    //PS_CONF1
    //Set the PS IRED on / off duty ratio setting
    VCNL4035X01_SET_PS_DUTY(VCNL4035X01_PS_DR_1_40);
    //Set the Persistence
    VCNL4035X01_SET_PS_PERS(VCNL4035X01_PS_PERS_1);
    //Set the Integration Time
    VCNL4035X01_SET_PS_IT(PS_IT);

    //PS_CONF2
    //Enable/Disable the Gesture Interrupt 
    VCNL4035X01_SET_GESTURE_INT(VCNL4035X01_PS_GESTURE_INT_EN);
    //Disable the Gesture Mode (Enable only after offset measurement)
    //If the mode be enabled now then only one channel will be measured
    VCNL4035X01_SET_GESTURE_MODE(VCNL4035X01_PS_GESTURE_MODE_DIS);
    //Set the Gain
    VCNL4035X01_SET_PS_Gain(VCNL4035X01_PS_GAIN_2_STEP);
    //Set the Output Bit Size
    VCNL4035X01_SET_PS_HD(VCNL4035X01_PS_HD_16Bits);
    //Set the Sensitivity Mode
    VCNL4035X01_SET_PS_NS(VCNL4035X01_PS_NS_2STEP_MODE_X4);
    //Set the Interrupt
    VCNL4035X01_SET_PS_INT(VCNL4035X01_PS_INT_CLOSE_AWAY);

    //PS_CONF3
    //Enable/Disable Low Current
    VCNL4035X01_SET_PS_LED_I_LOW(VCNL4035X01_PS_I_LOW_DIS);
    //Select IRED input to be Driven by the Internal Driver (For Gesture Mode not so important since all 3 channels will be read)
    VCNL4035X01_SET_PS_IRED_select(IRED_Channel);
    //Enable/Disable Smart Persistence
    VCNL4035X01_SET_PS_SMART_PERS(VCNL4035X01_PS_SMART_PERS_DIS);
    //Set Interrupt to Normal/Logic Mode
    VCNL4035X01_SET_PS_MS(VCNL4035X01_PS_MS_NORMAL);
    //Enable/Disable Sunlight Cancellation
    VCNL4035X01_SET_PS_SC_EN(VCNL4035X01_PS_SC_EN);
    //PS_AF has been set during initialization
    //PS_TRIG must be set before reading the proximity data

    //PS_MS
    //Set the Sunlight Cancellatin Current
    VCNL4035X01_SET_PS_SC_CUR(VCNL4035X01_PS_SC_CURx1);
    //Set the Sunlight Capability
    VCNL4035X01_SET_PS_SP(VCNL4035X01_PS_SPx1);
    //Set the Output of Sunlight Protect Mode
    VCNL4035X01_SET_PS_SPO(VCNL4035X01_PS_SPO_00);
    //Set the LED Current
    VCNL4035X01_SET_PS_LED_I(VCNL4035X01_PS_LED_I_200mA);

    //3.) Switch the sensor on
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_EN);
    //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
    CyDelay(10);

    //4.) Threshold Setting and Offset Measurement
    //Calculate Offset for all of the 3 channels
    for(int i=0; i<AverageCount;i++)
    { 
        //Channel 1
        //Select IRED input to be driven by the internal driver to calibrate
        VCNL4035X01_SET_PS_IRED_select(VCNL4035X01_PS_IRED_1);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
        //Enable trigger to start offset measurement
        VCNL4035X01_SET_PS_TRIG(VCNL4035X01_PS_TRIG_EN);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
        //Read offset data channel 1 and add as calibration value for offset
        CalibValue += VCNL4035X01_READ_Reg(VCNL4035X01_PS_DATA_1);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);

        //Channel 2
        //Select IRED input to be driven by the internal driver to calibrate
        VCNL4035X01_SET_PS_IRED_select(VCNL4035X01_PS_IRED_2);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
        //Enable trigger to start offset measurement
        VCNL4035X01_SET_PS_TRIG(VCNL4035X01_PS_TRIG_EN);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
        //Read offset data channel 2 and add as calibration value for offset
        CalibValue += VCNL4035X01_READ_Reg(VCNL4035X01_PS_DATA_2);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);

        //Channel 3
        //Select IRED input to be driven by the internal driver to calibrate
        VCNL4035X01_SET_PS_IRED_select(VCNL4035X01_PS_IRED_3);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
        //Enable trigger to start offset measurement
        VCNL4035X01_SET_PS_TRIG(VCNL4035X01_PS_TRIG_EN);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
        //Read offset data channel 3 and add as calibration value for offset
        CalibValue += VCNL4035X01_READ_Reg(VCNL4035X01_PS_DATA_3);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
    }

    //Calculate the average of the offset measurement from the 3 channels
    CalibValue /= 3*AverageCount;

    //Check whether the offset value is too high to be set in the cancellation register
    // 1T max offset value for the cancellation register is 2048
    if((PS_IT==VCNL4035X01_PS_IT_1T)||(PS_IT==VCNL4035X01_PS_IT_1_5T))Max_Offset = 2048;
    // 2T max offset value for the cancellation register is 4096
    if((PS_IT==VCNL4035X01_PS_IT_2T)||(PS_IT==VCNL4035X01_PS_IT_2_5T)||(PS_IT==VCNL4035X01_PS_IT_3T)||(PS_IT==VCNL4035X01_PS_IT_3_5T))Max_Offset = 4096;
    // 4T max offset value for the cancellation register is 8192
    if(PS_IT==VCNL4035X01_PS_IT_4T)Max_Offset = 8192;
    // 8T max offset value for the cancellation register is 16384
    if(PS_IT==VCNL4035X01_PS_IT_8T)Max_Offset = 16384;

    if(CalibValue>Max_Offset)
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500 + OffsetValue);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000 + OffsetValue);
    }
    else
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000);
    }

    //5.) Enable the Gesture Mode 
    //Enable/Disable the Gesture Mode
    VCNL4035X01_SET_GESTURE_MODE(VCNL4035X01_PS_GESTURE_MODE_EN);

    //Tag mode to be Gesture Mode
    mode_ps = 'G';

    while (I2C_1_MasterStatus()!=I2C_1_MODE_COMPLETE_XFER){}
    CyDelay(1000);
}

void AF_Mode()
{
    //1.)Initialization
    //Disable the PS
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_DIS);
    //Setting up AF Mode
    VCNL4035X01_SET_PS_AF(VCNL4035X01_PS_AF_EN);
    //PS_SD will be set before measuring the offset or after all of the parameters have been set up

    //2.) Setting up PS 
    //PS_CONF1
    //Set the PS IRED on / off duty ratio setting
    VCNL4035X01_SET_PS_DUTY(VCNL4035X01_PS_DR_1_40);
    //Set the Persistence
    VCNL4035X01_SET_PS_PERS(VCNL4035X01_PS_PERS_1);
    //Set the Integration Time
    VCNL4035X01_SET_PS_IT(PS_IT);

    //PS_CONF2
    //Enable/Disable the Gesture Interrupt 
    VCNL4035X01_SET_GESTURE_INT(VCNL4035X01_PS_GESTURE_INT_DIS);
    //Enable/Disable the Gesture Mode 
    VCNL4035X01_SET_GESTURE_MODE(VCNL4035X01_PS_GESTURE_MODE_DIS);
    //Set the Gain
    VCNL4035X01_SET_PS_Gain(VCNL4035X01_PS_GAIN_2_STEP);
    //Set the Output Bit Size
    VCNL4035X01_SET_PS_HD(VCNL4035X01_PS_HD_16Bits);
    //Set the Sensitivity Mode
    VCNL4035X01_SET_PS_NS(VCNL4035X01_PS_NS_2STEP_MODE_X4);
    //Set the Interrupt
    VCNL4035X01_SET_PS_INT(VCNL4035X01_PS_INT_CLOSE_AWAY);

    //PS_CONF3
    //Enable/Disable Low Current
    VCNL4035X01_SET_PS_LED_I_LOW(VCNL4035X01_PS_I_LOW_DIS);
    //Select IRED input to be Driven by the Internal Driver
    VCNL4035X01_SET_PS_IRED_select(IRED_Channel);
    //Enable/Disable Smart Persistence
    VCNL4035X01_SET_PS_SMART_PERS(VCNL4035X01_PS_SMART_PERS_DIS);
    //Set Interrupt to Normal/Logic Mode
    VCNL4035X01_SET_PS_MS(VCNL4035X01_PS_MS_NORMAL);
    //Enable/Disable Sunlight Cancellation
    VCNL4035X01_SET_PS_SC_EN(VCNL4035X01_PS_SC_EN);
    //PS_AF has been set during initialization
    //PS_TRIG must be set before reading the proximity data

    //PS_MS
    //Set the Sunlight Cancellatin Current
    VCNL4035X01_SET_PS_SC_CUR(VCNL4035X01_PS_SC_CURx1);
    //Set the Sunlight Capability
    VCNL4035X01_SET_PS_SP(VCNL4035X01_PS_SPx1);
    //Set the Output of Sunlight Protect Mode
    VCNL4035X01_SET_PS_SPO(VCNL4035X01_PS_SPO_00);
    //Set the LED Current
    VCNL4035X01_SET_PS_LED_I(VCNL4035X01_PS_LED_I_200mA);

    //3.) Switch the sensor on
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_EN);
    //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
    CyDelay(10);

    //4.) Threshold Setting and Offset Measurement
    for(int i=0;i<AverageCount;i++)
    {   
        //Enable trigger to start offset measurement
        VCNL4035X01_SET_PS_TRIG(VCNL4035X01_PS_TRIG_EN);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
        CalibValue += VCNL4035X01_READ_Reg(IRED_Channel_Offset_Read);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
    }

    //Calculate the average of the offset measurement
    CalibValue /= AverageCount;

    //Check whether the offset value is too high to have been set in the cancellation register
    // 1T max offset value for the cancellation register is 2048
    if((PS_IT==VCNL4035X01_PS_IT_1T)||(PS_IT==VCNL4035X01_PS_IT_1_5T))Max_Offset = 2048;
    // 2T max offset value for the cancellation register is 4096
    if((PS_IT==VCNL4035X01_PS_IT_2T)||(PS_IT==VCNL4035X01_PS_IT_2_5T)||(PS_IT==VCNL4035X01_PS_IT_3T)||(PS_IT==VCNL4035X01_PS_IT_3_5T))Max_Offset = 4096;
    // 4T max offset value for the cancellation register is 8192
    if(PS_IT==VCNL4035X01_PS_IT_4T)Max_Offset = 8192;
    // 8T max offset value for the cancellation register is 16384
    if(PS_IT==VCNL4035X01_PS_IT_8T)Max_Offset = 16384;
    if(CalibValue>Max_Offset)
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500 + OffsetValue);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000 + OffsetValue);
    }
    else
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000);
    }

    //Tag mode to be AF Mode (Also known as Continuous Forced Mode (Could also be used for single forced mode))
    mode_ps = 'A';

    while (I2C_1_MasterStatus()!=I2C_1_MODE_COMPLETE_XFER){}
    CyDelay(1000);
}

void Auto_Mode()
{
    //1.)Initialization
    //Disable the PS
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_DIS);
    //Setting up Auto Mode
    VCNL4035X01_SET_PS_AF(VCNL4035X01_PS_AUTO);
    //PS_SD will be set before measuring the offset or after all of the parameters have been set up

    //2.) Setting up PS 
    //PS_CONF1
    //Set the PS IRED on / off duty ratio setting
    VCNL4035X01_SET_PS_DUTY(VCNL4035X01_PS_DR_1_40);
    //Set the Persistence
    VCNL4035X01_SET_PS_PERS(VCNL4035X01_PS_PERS_1);
    //Set the Integration Time
    VCNL4035X01_SET_PS_IT(PS_IT);

    //PS_CONF2
    //Enable/Disable the Gesture Interrupt 
    VCNL4035X01_SET_GESTURE_INT(VCNL4035X01_PS_GESTURE_INT_DIS);
    //Enable/Disable the Gesture Mode 
    VCNL4035X01_SET_GESTURE_MODE(VCNL4035X01_PS_GESTURE_MODE_DIS);
    //Set the Gain
    VCNL4035X01_SET_PS_Gain(VCNL4035X01_PS_GAIN_2_STEP);
    //Set the Output Bit Size
    VCNL4035X01_SET_PS_HD(VCNL4035X01_PS_HD_16Bits);
    //Set the Sensitivity Mode
    VCNL4035X01_SET_PS_NS(VCNL4035X01_PS_NS_2STEP_MODE_X4);
    //Set the Interrupt
    VCNL4035X01_SET_PS_INT(VCNL4035X01_PS_INT_CLOSE_AWAY);

    //PS_CONF3
    //Enable/Disable Low Current
    VCNL4035X01_SET_PS_LED_I_LOW(VCNL4035X01_PS_I_LOW_DIS);
    //Select IRED input to be Driven by the Internal Driver
    VCNL4035X01_SET_PS_IRED_select(IRED_Channel);
    //Enable/Disable Smart Persistence
    VCNL4035X01_SET_PS_SMART_PERS(VCNL4035X01_PS_SMART_PERS_DIS);
    //Set Interrupt to Normal/Logic Mode
    VCNL4035X01_SET_PS_MS(VCNL4035X01_PS_MS_NORMAL);
    //Enable/Disable Sunlight Cancellation
    VCNL4035X01_SET_PS_SC_EN(VCNL4035X01_PS_SC_EN);
    //PS_AF has been set during initialization
    //PS_TRIG must be set before reading the proximity data

    //PS_MS
    //Set the Sunlight Cancellation Current
    VCNL4035X01_SET_PS_SC_CUR(VCNL4035X01_PS_SC_CURx1);
    //Set the Sunlight Capability
    VCNL4035X01_SET_PS_SP(VCNL4035X01_PS_SPx1);
    //Set the Output of Sunlight Protect Mode
    VCNL4035X01_SET_PS_SPO(VCNL4035X01_PS_SPO_00);
    //Set the LED Current
    VCNL4035X01_SET_PS_LED_I(VCNL4035X01_PS_LED_I_200mA);

    //3.) Switch the sensor on
    VCNL4035X01_SET_PS_SD(VCNL4035X01_PS_SD_EN);
    //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
    CyDelay(10);

    //4.) Threshold Setting and Offset Measurement
    for(int i=0;i<AverageCount;i++)
    {   
        CalibValue += VCNL4035X01_READ_Reg(IRED_Channel_Offset_Read);
        //Delay of 10 ms needs to be changed depending on the API of the µ-controller of use
        CyDelay(10);
    }

    //Calculate the average of the offset measurement
    CalibValue /= AverageCount;

    //Check whether the offset value is too high to have been set in the cancellation register
    // 1T max offset value for the cancellation register is 2048
    if((PS_IT==VCNL4035X01_PS_IT_1T)||(PS_IT==VCNL4035X01_PS_IT_1_5T))Max_Offset = 2048;
    // 2T max offset value for the cancellation register is 4096
    if((PS_IT==VCNL4035X01_PS_IT_2T)||(PS_IT==VCNL4035X01_PS_IT_2_5T)||(PS_IT==VCNL4035X01_PS_IT_3T)||(PS_IT==VCNL4035X01_PS_IT_3_5T))Max_Offset = 4096;
    // 4T max offset value for the cancellation register is 8192
    if(PS_IT==VCNL4035X01_PS_IT_4T)Max_Offset = 8192;
    // 8T max offset value for the cancellation register is 16384
    if(PS_IT==VCNL4035X01_PS_IT_8T)Max_Offset = 16384;
    if(CalibValue>Max_Offset)
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500 + OffsetValue);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000 + OffsetValue);
    }
    else
    {
        OffsetValue = CalibValue;
        //Set Cancellation register to eliminate offset
        VCNL4035X01_SET_PS_CANC(OffsetValue);
        //Set Low Threshold
        VCNL4035X01_SET_PS_LowThreshold(500);
        //set High Threshold
        VCNL4035X01_SET_PS_HighThreshold(1000);
    }

    //Tag mode to be Auto Mode/Self-timed
    mode_ps = 'S';

    while (I2C_1_MasterStatus()!=I2C_1_MODE_COMPLETE_XFER){}
	CyDelay(1000);
}